//
// huffman.test.cpp
//
// (c) Copyright 2000-2002 William A. McKee.  All rights reserved.
//
// Please send questions and comments to wamckee@msn.com
//
// Created April 27, 2000
//
// STL implementation of Huffman encoding.
//
// Compiled with VC++ 6.0: cl /GX huff.cpp
//
// Modification suggested by TiTi(tm), 28/04/2000
//   - make Print (renamed Encode) virtual
//
// Suggestion by NeilB (April 28, 2000)
//   - private data, protected accessors
//   - "IMHO typedefs like this obfuscate the code rather than make it 
//     clearer." [I tend to agree, so I removed the typedef. - WAM]
//
// December 7, 2002 - rewritten to use huffman.h
//

#include "huffman.h"

int cmp_Encoding (const void * a, const void * b)
{
    Encoding * aa = (Encoding *) a;
    Encoding * bb = (Encoding *) b;
    return aa->huffman_string < bb->huffman_string ? -1 :
           aa->huffman_string == bb->huffman_string ? 0 : 1;
}

int main (int argc, char * * argv)
{
    if (argc < 2)
    {
        char * default_data [] = { NULL,
            "15", "4", "17", "8", "30", "12", "41", "10", "6", "19", "27" };

        argc = sizeof (default_data) / sizeof (*default_data);
        argv = default_data;
    }

    std::vector<Node *> data;

    for (int i = 1; i < argc; i++)
    {
        data.push_back (new Leaf (atoi (argv [i]), i));
    }

    std::cout << "Huffman encoding:" << std::endl;

    Node * const root = BuildHuffman (data);

    if (root != NULL)
    {
        int cnt = 0;
        std::string str;
        Table<Encoding> table;

        root -> Encode (cnt, str, table);

        MakeCanonical (table);

        qsort (& table [0], cnt, sizeof (table [0]), cmp_Encoding);

        for (int j = 0; j < cnt; j++)
        {
            std::cout << table [j].huffman_code
                 << " - "
                 << table [j].huffman_string
                 << std::endl;
        }

        delete root;
    }

    return EXIT_SUCCESS;
}

